#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $max_level = 7;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";
my $path = Math::PlanePath::R5DragonCurve->new;

sub square_key {
  my ($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4) = @_;
  for (;;) {
    last if $x1 == min($x1,$x2,$x3,$x4)
      && $y1 == min($y1,$y2,$y3,$y4);
    ($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4) = ($x4,$y4, $x1,$y1, $x2,$y2, $x3,$y3);
  }
  return "$x1,$y1";
}

my %values;
foreach my $type ('GRQ',
                  # 'GRSH'
                 ) {
  my $factor = ($type eq 'GRQ' ? '1/4'
                : $type eq 'GRSH' ? '1/4'
                : die);

  printwrap_indent("% GP-DEFINE  ");
  printwrap("${type}_generated_samples = {[\n");
  $Printwrap = 2;
  foreach my $k (0 .. $max_level) {
    ### $k
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $count = 0;
    my $x_total = 0;
    my $y_total = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      next unless $path->_UNDOCUMENTED__n_segment_is_right_boundary($n);
      my ($x1,$y1) = $path->n_to_xy($n);

      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $x2 = $x1 + $dx;
      my $y2 = $y1 + $dy;

      ($dx,$dy) = xy_rotate_minus90($dx,$dy);
      my $x3 = $x2 + $dx;
      my $y3 = $y2 + $dy;

      ($dx,$dy) = xy_rotate_minus90($dx,$dy);
      my $x4 = $x3 + $dx;
      my $y4 = $y3 + $dy;

      if ($seen{square_key($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4)}++) {
        ### duplicate ...
        next;
      }
      ### at: "n=$n  $x1,$y1    $x2,$y2    $x3,$y3   $x4,$y4"

      if ($type eq 'GRQ') {
        $x_total += ($x1+$x2+$x3+$x4);
        $y_total += ($y1+$y2+$y3+$y4);

      } elsif ($type eq 'GRSH') {
        my $n = $path->xyxy_to_n_either($x2,$y2, $x3,$y3);
        if (defined $n && $n < $n_hi) {
          ### shortcut x4 to x3 ..
          $x_total += $x3 + $x4;
          $y_total += $y3 + $y4;
        } else {
          $x_total += $x3 + $x4;
          $y_total += $y3 + $y4;
        }
      }
      $count++;
    }
    my $comma = ($k == $max_level ? '' : ',');

    push @{$values{$type}->{'x'}}, $x_total;
    push @{$values{$type}->{'y'}}, $y_total;

    # my $gcd = gcd($x_total, gcd($y_total,$count));
    # $count /= $gcd;
    # $x_total /= $gcd;
    # $y_total /= $gcd;
    printwrap("$factor*1/$count*($x_total + I*$y_total)$comma\n");
  }
  $Printwrap = 0;
  printwrap("]};\n");
}
print "% end generated\n";

# foreach my $type ('GRQ','GRSH') {
#   print "X = [ ", join(',', @{$values{$type}->{'x'}}), "]\n";
#   print "Y = [ ", join(',', @{$values{$type}->{'y'}}), "]\n";
#   print "XW = [ ", join(',', @{$values{$type}->{'xw'}}), "]\n";
# }
# 
# foreach my $type ('GRQ','GRSH') {
#   my $factor = ($type eq 'GRQ' ? 6
#                 : $type eq 'GRSH' ? 4
#                 : die);
#   my $x_array = $values{$type}->{'x'};
#   my $y_array = $values{$type}->{'y'};
#   foreach my $i (0 .. min(4, $#$x_array)) {
#     my $x = $x_array->[$i];
#     my $y = $y_array->[$i];
#     if ($y == 0) {
#       printf "\\tfrac{%d}{$factor}, \\;\n", $x;
#     } else {
#       $y = sprintf "%+d", $y;
#       if (abs($y) == 1) {
#         $y =~ s/1//;
#       }
#       printf "\\tfrac{%d %s \\sqrt3 i}{$factor}, \\;\n", $x, $y;
#     }
#     $factor *= 2;
#   }
# }
# foreach my $type ('GRQ','GRSH') {
#   my $factor = ($type eq 'GRQ' ? 6
#                 : $type eq 'GRSH' ? 4
#                 : die);
#   my $x_array = $values{$type}->{'x'};
#   my $y_array = $values{$type}->{'y'};
#   foreach my $i (0 .. min(4, $#$x_array)) {
#     my $x = $x_array->[$i];
#     my $y = $y_array->[$i];
#     $y = sprintf "%+d", $y;
#     printf "%% Test-Pari  ${type}_exact($i) == 1/$factor*(%d %s*I)\n", $x, $y;
#     $factor *= 2;
#   }
# }

exit 0;
